module jp.ku6.windows.directx.dwrite;

import jp.ku6.windows.win32api.basetyps;
import jp.ku6.windows.win32api.winbase;
import jp.ku6.windows.win32api.windef;
import jp.ku6.windows.win32api.wingdi;
import jp.ku6.windows.win32api.unknwn;
import jp.ku6.windows.directx.dcommon;
import jp.ku6.windows.directx.d2d1;

enum DWRITE_FONT_FILE_TYPE
{
	DWRITE_FONT_FILE_TYPE_UNKNOWN,
	DWRITE_FONT_FILE_TYPE_CFF,
	DWRITE_FONT_FILE_TYPE_TRUETYPE,
	DWRITE_FONT_FILE_TYPE_TRUETYPE_COLLECTION,
	DWRITE_FONT_FILE_TYPE_TYPE1_PFM,
	DWRITE_FONT_FILE_TYPE_TYPE1_PFB,
	DWRITE_FONT_FILE_TYPE_VECTOR,
	DWRITE_FONT_FILE_TYPE_BITMAP
}

enum DWRITE_FONT_FACE_TYPE
{
	DWRITE_FONT_FACE_TYPE_CFF,
	DWRITE_FONT_FACE_TYPE_TRUETYPE,
	DWRITE_FONT_FACE_TYPE_TRUETYPE_COLLECTION,
	DWRITE_FONT_FACE_TYPE_TYPE1,
	DWRITE_FONT_FACE_TYPE_VECTOR,
	DWRITE_FONT_FACE_TYPE_BITMAP,
	DWRITE_FONT_FACE_TYPE_UNKNOWN
}

enum DWRITE_FONT_SIMULATIONS
{
	DWRITE_FONT_SIMULATIONS_NONE    = 0x0000,
	DWRITE_FONT_SIMULATIONS_BOLD    = 0x0001,
	DWRITE_FONT_SIMULATIONS_OBLIQUE = 0x0002
}

enum DWRITE_FONT_WEIGHT
{
	DWRITE_FONT_WEIGHT_THIN        = 100,
	DWRITE_FONT_WEIGHT_EXTRA_LIGHT = 200,
	DWRITE_FONT_WEIGHT_ULTRA_LIGHT = 200,
	DWRITE_FONT_WEIGHT_LIGHT       = 300,
	DWRITE_FONT_WEIGHT_NORMAL      = 400,
	DWRITE_FONT_WEIGHT_REGULAR     = 400,
	DWRITE_FONT_WEIGHT_MEDIUM      = 500,
	DWRITE_FONT_WEIGHT_DEMI_BOLD   = 600,
	DWRITE_FONT_WEIGHT_SEMI_BOLD   = 600,
	DWRITE_FONT_WEIGHT_BOLD        = 700,
	DWRITE_FONT_WEIGHT_EXTRA_BOLD  = 800,
	DWRITE_FONT_WEIGHT_ULTRA_BOLD  = 800,
	DWRITE_FONT_WEIGHT_BLACK       = 900,
	DWRITE_FONT_WEIGHT_HEAVY       = 900,
	DWRITE_FONT_WEIGHT_EXTRA_BLACK = 950,
	DWRITE_FONT_WEIGHT_ULTRA_BLACK = 950
}

enum DWRITE_FONT_STRETCH
{
	DWRITE_FONT_STRETCH_UNDEFINED       = 0,
	DWRITE_FONT_STRETCH_ULTRA_CONDENSED = 1,
	DWRITE_FONT_STRETCH_EXTRA_CONDENSED = 2,
	DWRITE_FONT_STRETCH_CONDENSED       = 3,
	DWRITE_FONT_STRETCH_SEMI_CONDENSED  = 4,
	DWRITE_FONT_STRETCH_NORMAL          = 5,
	DWRITE_FONT_STRETCH_MEDIUM          = 5,
	DWRITE_FONT_STRETCH_SEMI_EXPANDED   = 6,
	DWRITE_FONT_STRETCH_EXPANDED        = 7,
	DWRITE_FONT_STRETCH_EXTRA_EXPANDED  = 8,
	DWRITE_FONT_STRETCH_ULTRA_EXPANDED  = 9
}

enum DWRITE_FONT_STYLE
{
	DWRITE_FONT_STYLE_NORMAL,
	DWRITE_FONT_STYLE_OBLIQUE,
	DWRITE_FONT_STYLE_ITALIC
}

enum DWRITE_INFORMATIONAL_STRING_ID
{
	DWRITE_INFORMATIONAL_STRING_NONE,
	DWRITE_INFORMATIONAL_STRING_COPYRIGHT_NOTICE,
	DWRITE_INFORMATIONAL_STRING_VERSION_STRINGS,
	DWRITE_INFORMATIONAL_STRING_TRADEMARK,
	DWRITE_INFORMATIONAL_STRING_MANUFACTURER,
	DWRITE_INFORMATIONAL_STRING_DESIGNER,
	DWRITE_INFORMATIONAL_STRING_DESIGNER_URL,
	DWRITE_INFORMATIONAL_STRING_DESCRIPTION,
	DWRITE_INFORMATIONAL_STRING_FONT_VENDOR_URL,
	DWRITE_INFORMATIONAL_STRING_LICENSE_DESCRIPTION,
	DWRITE_INFORMATIONAL_STRING_LICENSE_INFO_URL,
	DWRITE_INFORMATIONAL_STRING_WIN32_FAMILY_NAMES,
	DWRITE_INFORMATIONAL_STRING_WIN32_SUBFAMILY_NAMES,
	DWRITE_INFORMATIONAL_STRING_PREFERRED_FAMILY_NAMES,
	DWRITE_INFORMATIONAL_STRING_PREFERRED_SUBFAMILY_NAMES,
	DWRITE_INFORMATIONAL_STRING_SAMPLE_TEXT
}

struct DWRITE_FONT_METRICS
{
	UINT16 designUnitsPerEm;
	UINT16 ascent;
	UINT16 descent;
	INT16 lineGap;
	UINT16 capHeight;
	UINT16 xHeight;
	INT16 underlinePosition;
	UINT16 underlineThickness;
	INT16 strikethroughPosition;
	UINT16 strikethroughThickness;
}

struct DWRITE_GLYPH_METRICS
{
	INT32 leftSideBearing;
	UINT32 advanceWidth;
	INT32 rightSideBearing;
	INT32 topSideBearing;
	UINT32 advanceHeight;
	INT32 bottomSideBearing;
	INT32 verticalOriginY;
}

struct DWRITE_GLYPH_OFFSET
{
	FLOAT advanceOffset;
	FLOAT ascenderOffset;
}

enum DWRITE_FACTORY_TYPE
{
	DWRITE_FACTORY_TYPE_SHARED,
	DWRITE_FACTORY_TYPE_ISOLATED
}


template DWRITE_MAKE_OPENTYPE_TAG(UINT8 a, UINT8 b, UINT8 c, UINT8 d)
{
	immutable UINT32 DWRITE_MAKE_OPENTYPE_TAG = (d << 24) | (c << 16) | (b << 8) | a;
}

extern (Windows)
{
	interface IDWriteFontFileLoader : IUnknown
	{
		HRESULT CreateStreamFromKey(const(void)* fontFileReferenceKey, UINT32 fontFileReferenceKeySize, IDWriteFontFileStream* fontFileStream);
	}

	interface IDWriteLocalFontFileLoader : IDWriteFontFileLoader
	{
		HRESULT GetFilePathLengthFromKey(const(void)* fontFileReferenceKey, UINT32 fontFileReferenceKeySize, UINT32* filePathLength);
		HRESULT GetFilePathFromKey(const(void)* fontFileReferenceKey, UINT32 fontFileReferenceKeySize, WCHAR* filePath, UINT32 filePathSize);
		HRESULT GetLastWriteTimeFromKey(const(void)* fontFileReferenceKey, UINT32 fontFileReferenceKeySize, FILETIME* lastWriteTime);
	}

	interface IDWriteFontFileStream : IUnknown
	{
		HRESULT ReadFileFragment(const(void)** fragmentStart, UINT64 fileOffset, UINT64 fragmentSize, void** fragmentContext);
		void ReleaseFileFragment(void* fragmentContext);
		HRESULT GetFileSize(UINT64* fileSize);
		HRESULT GetLastWriteTime(UINT64* lastWriteTime);
	}

	interface IDWriteFontFile : IUnknown
	{
		HRESULT GetReferenceKey(void** fontFileReferenceKey, UINT32* fontFileReferenceKeySize);
		HRESULT GetLoader(IDWriteFontFileLoader* fontFileLoader);
		HRESULT Analyze(BOOL* isSupportedFontType, DWRITE_FONT_FILE_TYPE* fontFileType, DWRITE_FONT_FACE_TYPE* fontFaceType, UINT32* numberOfFaces);
	}
}

enum DWRITE_PIXEL_GEOMETRY
{
	DWRITE_PIXEL_GEOMETRY_FLAT,
	DWRITE_PIXEL_GEOMETRY_RGB,
	DWRITE_PIXEL_GEOMETRY_BGR
}

enum DWRITE_RENDERING_MODE
{
	DWRITE_RENDERING_MODE_DEFAULT,
	DWRITE_RENDERING_MODE_ALIASED,
	DWRITE_RENDERING_MODE_CLEARTYPE_GDI_CLASSIC,
	DWRITE_RENDERING_MODE_CLEARTYPE_GDI_NATURAL,
	DWRITE_RENDERING_MODE_CLEARTYPE_NATURAL,
	DWRITE_RENDERING_MODE_CLEARTYPE_NATURAL_SYMMETRIC,
	DWRITE_RENDERING_MODE_OUTLINE
}

struct DWRITE_MATRIX
{
	FLOAT m11;
	FLOAT m12;
	FLOAT m21;
	FLOAT m22;
	FLOAT dx;
	FLOAT dy;
}

extern (Windows)
{
	interface IDWriteRenderingParams : IUnknown
	{
		FLOAT GetGamma();
		FLOAT GetEnhancedContrast();
		FLOAT GetClearTypeLevel();
		DWRITE_PIXEL_GEOMETRY GetPixelGeometry();
		DWRITE_RENDERING_MODE GetRenderingMode();
	}

	alias ID2D1SimplifiedGeometrySink IDWriteGeometrySink;

	interface IDWriteFontFace : IUnknown
	{
		DWRITE_FONT_FACE_TYPE GetType();
		HRESULT GetFiles(UINT32* numberOfFiles, IDWriteFontFile* fontFiles);
		UINT32 GetIndex();
		DWRITE_FONT_SIMULATIONS GetSimulations();
		void GetMetrics(DWRITE_FONT_METRICS* fontFaceMetrics);
		UINT16 GetGlyphCount();
		HRESULT GetDesignGlyphMetrics(const(UINT16)* glyphIndices, UINT32 glyphCount, DWRITE_GLYPH_METRICS* glyphMetrics, BOOL isSideways = FALSE);
		HRESULT GetGlyphIndices(const(UINT32)* codePoints, UINT32 codePointCount, UINT16* glyphIndices);
		HRESULT TryGetFontTable(UINT32 openTypeTableTag, void** tableData, UINT32* tableSize, void** tableContext, BOOL* exists);
		void ReleaseFontTable(void* tableContext);
		HRESULT GetGlyphRunOutline(FLOAT emSize, const(UINT16)* glyphIndices, const(FLOAT)* glyphAdvances, const(DWRITE_GLYPH_OFFSET)* glyphOffsets, UINT32 glyphCount, BOOL isSideways, BOOL isRightToLeft, IDWriteGeometrySink geometrySink);
		HRESULT GetRecommendedRenderingMode(FLOAT emSize, FLOAT pixelsPerDip, DWRITE_MEASURING_MODE measuringMode, IDWriteRenderingParams renderingParams, DWRITE_RENDERING_MODE* renderingMode);
		HRESULT GetGdiCompatibleMetrics(FLOAT emSize, FLOAT pixelsPerDip, const(DWRITE_MATRIX)* transform, DWRITE_FONT_METRICS* fontFaceMetrics);
		HRESULT GetGdiCompatibleGlyphMetrics(FLOAT emSize, FLOAT pixelsPerDip, const(DWRITE_MATRIX)* transform, BOOL useGdiNatural, const(UINT16)* glyphIndices, UINT32 glyphCount, DWRITE_GLYPH_METRICS* glyphMetrics, BOOL isSideways = FALSE);
	}

	interface IDWriteFontCollectionLoader : IUnknown
	{
		HRESULT CreateEnumeratorFromKey(IDWriteFactory factory, const(void)* collectionKey, UINT32 collectionKeySize, IDWriteFontFileEnumerator* fontFileEnumerator);
	}

	interface IDWriteFontFileEnumerator : IUnknown
	{
		HRESULT MoveNext(BOOL* hasCurrentFile);
		HRESULT GetCurrentFontFile(IDWriteFontFile* fontFile);
	}

	interface IDWriteLocalizedStrings : IUnknown
	{
		UINT32 GetCount();
		HRESULT FindLocaleName(const(WCHAR)* localeName, UINT32* index, BOOL* exists);
		HRESULT GetLocaleNameLength(UINT32 index, UINT32* length);
		HRESULT GetLocaleName(UINT32 index, WCHAR* localeName, UINT32 size);
		HRESULT GetStringLength(UINT32 index, UINT32* length);
		HRESULT GetString(UINT32 index, WCHAR* stringBuffer, UINT32 size);
	}

	interface IDWriteFontCollection : IUnknown
	{
		UINT32 GetFontFamilyCount();
		HRESULT GetFontFamily(UINT32 index, IDWriteFontFamily* fontFamily);
		HRESULT FindFamilyName(const(WCHAR)* familyName, UINT32* index, BOOL* exists);
		HRESULT GetFontFromFontFace(IDWriteFontFace fontFace, IDWriteFont* font);
	}

	interface IDWriteFontList : IUnknown
	{
		HRESULT GetFontCollection(IDWriteFontCollection* fontCollection);
		UINT32 GetFontCount();
		HRESULT GetFont(UINT32 index, IDWriteFont* font);
	}

	interface IDWriteFontFamily : IDWriteFontList
	{
		HRESULT GetFamilyNames(IDWriteLocalizedStrings* names);
		HRESULT GetFirstMatchingFont(DWRITE_FONT_WEIGHT weight, DWRITE_FONT_STRETCH stretch, DWRITE_FONT_STYLE style, IDWriteFont* matchingFont);
		HRESULT GetMatchingFonts(DWRITE_FONT_WEIGHT weight, DWRITE_FONT_STRETCH stretch, DWRITE_FONT_STYLE style, IDWriteFontList* matchingFonts);
	}

	interface IDWriteFont : IUnknown
	{
		HRESULT GetFontFamily(IDWriteFontFamily* fontFamily);
		DWRITE_FONT_WEIGHT GetWeight();
		DWRITE_FONT_STRETCH GetStretch();
		DWRITE_FONT_STYLE GetStyle();
		BOOL IsSymbolFont();
		HRESULT GetFaceNames(IDWriteLocalizedStrings* names);
		HRESULT GetInformationalStrings(DWRITE_INFORMATIONAL_STRING_ID informationalStringID, IDWriteLocalizedStrings* informationalStrings, BOOL* exists);
		DWRITE_FONT_SIMULATIONS GetSimulations();
		void GetMetrics(DWRITE_FONT_METRICS* fontMetrics);
		HRESULT HasCharacter(UINT32 unicodeValue, BOOL* exists);
		HRESULT CreateFontFace(IDWriteFontFace* fontFace);
	}
}

enum DWRITE_READING_DIRECTION
{
	DWRITE_READING_DIRECTION_LEFT_TO_RIGHT,
	DWRITE_READING_DIRECTION_RIGHT_TO_LEFT
}

enum DWRITE_FLOW_DIRECTION
{
	DWRITE_FLOW_DIRECTION_TOP_TO_BOTTOM
}

enum DWRITE_TEXT_ALIGNMENT
{
	DWRITE_TEXT_ALIGNMENT_LEADING,
	DWRITE_TEXT_ALIGNMENT_TRAILING,
	DWRITE_TEXT_ALIGNMENT_CENTER
}

enum DWRITE_PARAGRAPH_ALIGNMENT
{
	DWRITE_PARAGRAPH_ALIGNMENT_NEAR,
	DWRITE_PARAGRAPH_ALIGNMENT_FAR,
	DWRITE_PARAGRAPH_ALIGNMENT_CENTER
}

enum DWRITE_WORD_WRAPPING
{
	DWRITE_WORD_WRAPPING_WRAP,
	DWRITE_WORD_WRAPPING_NO_WRAP
}

enum DWRITE_LINE_SPACING_METHOD
{
	DWRITE_LINE_SPACING_METHOD_DEFAULT,
	DWRITE_LINE_SPACING_METHOD_UNIFORM
}

enum DWRITE_TRIMMING_GRANULARITY
{
	DWRITE_TRIMMING_GRANULARITY_NONE,
	DWRITE_TRIMMING_GRANULARITY_CHARACTER,
	DWRITE_TRIMMING_GRANULARITY_WORD	
}

enum DWRITE_FONT_FEATURE_TAG
{
	DWRITE_FONT_FEATURE_TAG_ALTERNATIVE_FRACTIONS           = 0x63726661,
	DWRITE_FONT_FEATURE_TAG_PETITE_CAPITALS_FROM_CAPITALS   = 0x63703263,
	DWRITE_FONT_FEATURE_TAG_SMALL_CAPITALS_FROM_CAPITALS    = 0x63733263,
	DWRITE_FONT_FEATURE_TAG_CONTEXTUAL_ALTERNATES           = 0x746c6163,
	DWRITE_FONT_FEATURE_TAG_CASE_SENSITIVE_FORMS            = 0x65736163,
	DWRITE_FONT_FEATURE_TAG_GLYPH_COMPOSITION_DECOMPOSITION = 0x706d6363,
	DWRITE_FONT_FEATURE_TAG_CONTEXTUAL_LIGATURES            = 0x67696c63,
	DWRITE_FONT_FEATURE_TAG_CAPITAL_SPACING                 = 0x70737063,
	DWRITE_FONT_FEATURE_TAG_CONTEXTUAL_SWASH                = 0x68777363,
	DWRITE_FONT_FEATURE_TAG_CURSIVE_POSITIONING             = 0x73727563,
	DWRITE_FONT_FEATURE_TAG_DEFAULT                         = 0x746c6664,
	DWRITE_FONT_FEATURE_TAG_DISCRETIONARY_LIGATURES         = 0x67696c64,
	DWRITE_FONT_FEATURE_TAG_EXPERT_FORMS                    = 0x74707865,
	DWRITE_FONT_FEATURE_TAG_FRACTIONS                       = 0x63617266,
	DWRITE_FONT_FEATURE_TAG_FULL_WIDTH                      = 0x64697766,
	DWRITE_FONT_FEATURE_TAG_HALF_FORMS                      = 0x666c6168,
	DWRITE_FONT_FEATURE_TAG_HALANT_FORMS                    = 0x6e6c6168,
	DWRITE_FONT_FEATURE_TAG_ALTERNATE_HALF_WIDTH            = 0x746c6168,
	DWRITE_FONT_FEATURE_TAG_HISTORICAL_FORMS                = 0x74736968,
	DWRITE_FONT_FEATURE_TAG_HORIZONTAL_KANA_ALTERNATES      = 0x616e6b68,
	DWRITE_FONT_FEATURE_TAG_HISTORICAL_LIGATURES            = 0x67696c68,
	DWRITE_FONT_FEATURE_TAG_HALF_WIDTH                      = 0x64697768,
	DWRITE_FONT_FEATURE_TAG_HOJO_KANJI_FORMS                = 0x6f6a6f68,
	DWRITE_FONT_FEATURE_TAG_JIS04_FORMS                     = 0x3430706a,
	DWRITE_FONT_FEATURE_TAG_JIS78_FORMS                     = 0x3837706a,
	DWRITE_FONT_FEATURE_TAG_JIS83_FORMS                     = 0x3338706a,
	DWRITE_FONT_FEATURE_TAG_JIS90_FORMS                     = 0x3039706a,
	DWRITE_FONT_FEATURE_TAG_KERNING                         = 0x6e72656b,
	DWRITE_FONT_FEATURE_TAG_STANDARD_LIGATURES              = 0x6167696c,
	DWRITE_FONT_FEATURE_TAG_LINING_FIGURES                  = 0x6d756e6c,
	DWRITE_FONT_FEATURE_TAG_LOCALIZED_FORMS                 = 0x6c636f6c,
	DWRITE_FONT_FEATURE_TAG_MARK_POSITIONING                = 0x6b72616d,
	DWRITE_FONT_FEATURE_TAG_MATHEMATICAL_GREEK              = 0x6b72676d,
	DWRITE_FONT_FEATURE_TAG_MARK_TO_MARK_POSITIONING        = 0x6b6d6b6d,
	DWRITE_FONT_FEATURE_TAG_ALTERNATE_ANNOTATION_FORMS      = 0x746c616e,
	DWRITE_FONT_FEATURE_TAG_NLC_KANJI_FORMS                 = 0x6b636c6e,
	DWRITE_FONT_FEATURE_TAG_OLD_STYLE_FIGURES               = 0x6d756e6f,
	DWRITE_FONT_FEATURE_TAG_ORDINALS                        = 0x6e64726f,
	DWRITE_FONT_FEATURE_TAG_PROPORTIONAL_ALTERNATE_WIDTH    = 0x746c6170,
	DWRITE_FONT_FEATURE_TAG_PETITE_CAPITALS                 = 0x70616370,
	DWRITE_FONT_FEATURE_TAG_PROPORTIONAL_FIGURES            = 0x6d756e70,
	DWRITE_FONT_FEATURE_TAG_PROPORTIONAL_WIDTHS             = 0x64697770,
	DWRITE_FONT_FEATURE_TAG_QUARTER_WIDTHS                  = 0x64697771,
	DWRITE_FONT_FEATURE_TAG_REQUIRED_LIGATURES              = 0x67696c72,
	DWRITE_FONT_FEATURE_TAG_RUBY_NOTATION_FORMS             = 0x79627572,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_ALTERNATES            = 0x746c6173,
	DWRITE_FONT_FEATURE_TAG_SCIENTIFIC_INFERIORS            = 0x666e6973,
	DWRITE_FONT_FEATURE_TAG_SMALL_CAPITALS                  = 0x70636d73,
	DWRITE_FONT_FEATURE_TAG_SIMPLIFIED_FORMS                = 0x6c706d73,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_1                 = 0x31307373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_2                 = 0x32307373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_3                 = 0x33307373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_4                 = 0x34307373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_5                 = 0x35307373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_6                 = 0x36307373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_7                 = 0x37307373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_8                 = 0x38307373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_9                 = 0x39307373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_10                = 0x30317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_11                = 0x31317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_12                = 0x32317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_13                = 0x33317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_14                = 0x34317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_15                = 0x35317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_16                = 0x36317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_17                = 0x37317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_18                = 0x38317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_19                = 0x39317373,
	DWRITE_FONT_FEATURE_TAG_STYLISTIC_SET_20                = 0x30327373,
	DWRITE_FONT_FEATURE_TAG_SUBSCRIPT                       = 0x73627573,
	DWRITE_FONT_FEATURE_TAG_SUPERSCRIPT                     = 0x73707573,
	DWRITE_FONT_FEATURE_TAG_SWASH                           = 0x68737773,
	DWRITE_FONT_FEATURE_TAG_TITLING                         = 0x6c746974,
	DWRITE_FONT_FEATURE_TAG_TRADITIONAL_NAME_FORMS          = 0x6d616e74,
	DWRITE_FONT_FEATURE_TAG_TABULAR_FIGURES                 = 0x6d756e74,
	DWRITE_FONT_FEATURE_TAG_TRADITIONAL_FORMS               = 0x64617274,
	DWRITE_FONT_FEATURE_TAG_THIRD_WIDTHS                    = 0x64697774,
	DWRITE_FONT_FEATURE_TAG_UNICASE                         = 0x63696e75,
	DWRITE_FONT_FEATURE_TAG_SLASHED_ZERO                    = 0x6f72657a,
}

struct DWRITE_TEXT_RANGE
{
	UINT32 startPosition;
	UINT32 length;
}

struct DWRITE_FONT_FEATURE
{
	DWRITE_FONT_FEATURE_TAG nameTag;
	UINT32 parameter;
}

struct DWRITE_TYPOGRAPHIC_FEATURES
{
	DWRITE_FONT_FEATURE* features;
	UINT32 featureCount;
}

struct DWRITE_TRIMMING
{
	DWRITE_TRIMMING_GRANULARITY granularity;
	UINT32 delimiter;
	UINT32 delimiterCount;
}

extern (Windows)
{
	interface IDWriteTextFormat : IUnknown
	{
		HRESULT SetTextAlignment(DWRITE_TEXT_ALIGNMENT textAlignment);
		HRESULT SetParagraphAlignment(DWRITE_PARAGRAPH_ALIGNMENT paragraphAlignment);
		HRESULT SetWordWrapping(DWRITE_WORD_WRAPPING wordWrapping);
		HRESULT SetReadingDirection(DWRITE_READING_DIRECTION readingDirection);
		HRESULT SetFlowDirection(DWRITE_FLOW_DIRECTION flowDirection);
		HRESULT SetIncrementalTabStop(FLOAT incrementalTabStop);
		HRESULT SetTrimming(const(DWRITE_TRIMMING)* trimmingOptions, IDWriteInlineObject trimmingSign);
		HRESULT SetLineSpacing(DWRITE_LINE_SPACING_METHOD lineSpacingMethod, FLOAT lineSpacing, FLOAT baseline);
		DWRITE_TEXT_ALIGNMENT GetTextAlignment();
		DWRITE_PARAGRAPH_ALIGNMENT GetParagraphAlignment();
		DWRITE_WORD_WRAPPING GetWordWrapping();
		DWRITE_READING_DIRECTION GetReadingDirection();
		DWRITE_FLOW_DIRECTION GetFlowDirection();
		FLOAT GetIncrementalTabStop();
		HRESULT GetTrimming(DWRITE_TRIMMING* trimmingOptions, IDWriteInlineObject* trimmingSign);
		HRESULT GetLineSpacing(DWRITE_LINE_SPACING_METHOD* lineSpacingMethod, FLOAT* lineSpacing, FLOAT* baseline);
		HRESULT GetFontCollection(IDWriteFontCollection* fontCollection);
		UINT32 GetFontFamilyNameLength();
		HRESULT GetFontFamilyName(WCHAR* fontFamilyName, UINT32 nameSize);
		DWRITE_FONT_WEIGHT GetFontWeight();
		DWRITE_FONT_STYLE GetFontStyle();
		DWRITE_FONT_STRETCH GetFontStretch();
		FLOAT GetFontSize();
		UINT32 GetLocaleNameLength();
		HRESULT GetLocaleName(WCHAR* localeName, UINT32 nameSize);
	}

	interface IDWriteTypography : IUnknown
	{
		HRESULT AddFontFeature(DWRITE_FONT_FEATURE fontFeature);
		UINT32 GetFontFeatureCount();
		HRESULT GetFontFeature(UINT32 fontFeatureIndex, DWRITE_FONT_FEATURE* fontFeature);
	}
}

enum DWRITE_SCRIPT_SHAPES
{
	DWRITE_SCRIPT_SHAPES_DEFAULT   = 0,
	DWRITE_SCRIPT_SHAPES_NO_VISUAL = 1
}

struct DWRITE_SCRIPT_ANALYSIS
{
	UINT16 script;
	DWRITE_SCRIPT_SHAPES shapes;
}

enum DWRITE_BREAK_CONDITION
{
	DWRITE_BREAK_CONDITION_NEUTRAL,
	DWRITE_BREAK_CONDITION_CAN_BREAK,
	DWRITE_BREAK_CONDITION_MAY_NOT_BREAK,
	DWRITE_BREAK_CONDITION_MUST_BREAK
}

struct DWRITE_LINE_BREAKPOINT
{
	UINT8 bitfield;

	UINT8 breakConditionBefore()
	{
		return 0b00000011 & bitfield;
	}

	void breakConditionBefore(UINT8 a)
	{
		bitfield = (bitfield & 0b11111100) | (0b00000011 & a);
	}

	UINT8 breakConditionAfter()
	{
		return 0b00001100 & bitfield;
	}

	void breakConditionAfter(UINT8 a)
	{
		bitfield = (bitfield & 0b11110011) | (0b00001100 & a);
	}

	UINT8 isWhitespace()
	{
		return 0b00010000 & bitfield;
	}

	void isWhitespace(UINT8 a)
	{
		bitfield = (bitfield & 0b11101111) | (0b00010000 & a);
	}

	UINT8 isSoftHyphen()
	{
		return 0b00100000 & bitfield;
	}

	void isSoftHyphen(UINT8 a)
	{
		bitfield = (bitfield & 0b11011111) | (0b00100000 & a);
	}

	UINT8 padding()
	{
		return 0b11000000 & bitfield;
	}

	void padding(UINT8 a)
	{
		bitfield = (bitfield & 0b00111111) | (0b11000000 & a);
	}
}

enum DWRITE_NUMBER_SUBSTITUTION_METHOD
{
	DWRITE_NUMBER_SUBSTITUTION_METHOD_FROM_CULTURE,
	DWRITE_NUMBER_SUBSTITUTION_METHOD_CONTEXTUAL,
	DWRITE_NUMBER_SUBSTITUTION_METHOD_NONE,
	DWRITE_NUMBER_SUBSTITUTION_METHOD_NATIONAL,
	DWRITE_NUMBER_SUBSTITUTION_METHOD_TRADITIONAL
}

extern (Windows)
{
	interface IDWriteNumberSubstitution : IUnknown
	{
	}
}

struct DWRITE_SHAPING_TEXT_PROPERTIES
{
	UINT16 bitfield;

	UINT16 isShapedAlone()
	{
		return 0b0000000000000001 & bitfield;
	}

	void isShapedAlone(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111111111110) | (0b0000000000000001 & a);
	}

	UINT16 reserved()
	{
		return 0b1111111111111110 & bitfield;
	}

	void reserved(UINT16 a)
	{
		bitfield = (bitfield & 0b0000000000000001) | (0b1111111111111110 & a);
	}
}

struct DWRITE_SHAPING_GLYPH_PROPERTIES
{
	UINT16 bitfield;

	UINT16 justification()
	{
		return 0b0000000000001111 & bitfield;
	}

	void justification(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111111110000) | (0b0000000000001111 & a);
	}

	UINT16 isClusterStart()
	{
		return 0b0000000000010000 & bitfield;
	}

	void isClusterStart(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111111101111) | (0b0000000000010000 & a);
	}

	UINT16 isDiacritic()
	{
		return 0b0000000000100000 & bitfield;
	}

	void isDiacritic(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111111011111) | (0b0000000000100000 & a);
	}

	UINT16 isZeroWidthSpace()
	{
		return 0b0000000001000000 & bitfield;
	}

	void isZeroWidthSpace(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111110111111) | (0b0000000001000000 & a);
	}

	UINT16 reserved()
	{
		return 0b1111111110000000 & bitfield;
	}

	void reserved(UINT16 a)
	{
		bitfield = (bitfield & 0b0000000001111111) | (0b1111111110000000 & a);
	}
}

extern (Windows)
{
	interface IDWriteTextAnalysisSource : IUnknown
	{
		HRESULT GetTextAtPosition(UINT32 textPosition, WCHAR** textString, UINT32* textLength);
		HRESULT GetTextBeforePosition(UINT32 textPosition, WCHAR** textString, UINT32* textLength);
		DWRITE_READING_DIRECTION GetParagraphReadingDirection();
		HRESULT GetLocaleName(UINT32 textPosition, UINT32* textLength, WCHAR** localeName);
		HRESULT GetNumberSubstitution(UINT32 textPosition, UINT32* textLength, IDWriteNumberSubstitution* numberSubstitution);
	}

	interface IDWriteTextAnalysisSink : IUnknown
	{
		HRESULT SetScriptAnalysis(UINT32 textPosition, UINT32 textLength, const(DWRITE_SCRIPT_ANALYSIS)* scriptAnalysis);
		HRESULT SetLineBreakpoints(UINT32 textPosition, UINT32 textLength, const(DWRITE_LINE_BREAKPOINT)* lineBreakpoints);
		HRESULT SetBidiLevel(UINT32 textPosition, UINT32 textLength, UINT8 explicitLevel, UINT8 resolvedLevel);
		HRESULT SetNumberSubstitution(UINT32 textPosition, UINT32 textLength, IDWriteNumberSubstitution numberSubstitution);
	}

	interface IDWriteTextAnalyzer : IUnknown
	{
		HRESULT AnalyzeScript(IDWriteTextAnalysisSource analysisSource, UINT32 textPosition, UINT32 textLength, IDWriteTextAnalysisSink analysisSink);
		HRESULT AnalyzeBidi(IDWriteTextAnalysisSource analysisSource, UINT32 textPosition, UINT32 textLength, IDWriteTextAnalysisSink analysisSink);
		HRESULT AnalyzeNumberSubstitution(IDWriteTextAnalysisSource analysisSource, UINT32 textPosition, UINT32 textLength, IDWriteTextAnalysisSink analysisSink);
		HRESULT AnalyzeLineBreakpoints(IDWriteTextAnalysisSource analysisSource, UINT32 textPosition, UINT32 textLength, IDWriteTextAnalysisSink analysisSink);
		HRESULT GetGlyphs(const(WCHAR)* textString, UINT32 textLength, IDWriteFontFace fontFace, BOOL isSideways, BOOL isRightToLeft, const(DWRITE_SCRIPT_ANALYSIS)* scriptAnalysis, const(WCHAR)* localeName, IDWriteNumberSubstitution numberSubstitution, const(DWRITE_TYPOGRAPHIC_FEATURES)** features, const(UINT32)* featureRangeLengths, UINT32 featureRanges, UINT32 maxGlyphCount, UINT16* clusterMap, DWRITE_SHAPING_TEXT_PROPERTIES* textProps, UINT16* glyphIndices, DWRITE_SHAPING_GLYPH_PROPERTIES* glyphProps, UINT32* actualGlyphCount);
		HRESULT GetGlyphPlacements(const(WCHAR)* textString, const(UINT16)* clusterMap, DWRITE_SHAPING_TEXT_PROPERTIES* textProps, UINT32 textLength, const(UINT16)* glyphIndices, const(DWRITE_SHAPING_GLYPH_PROPERTIES)* glyphProps, UINT32 glyphCount, IDWriteFontFace fontFace, FLOAT fontEmSize, BOOL isSideways, BOOL isRightToLeft, const(DWRITE_SCRIPT_ANALYSIS)* scriptAnalysis, const(WCHAR)* localeName, const(DWRITE_TYPOGRAPHIC_FEATURES)** features, const(UINT32)* featureRangeLengths, UINT32 featureRanges, FLOAT* glyphAdvances, DWRITE_GLYPH_OFFSET* glyphOffsets);
		HRESULT GetGdiCompatibleGlyphPlacements(const(WCHAR)* textString, const(UINT16)* clusterMap, DWRITE_SHAPING_TEXT_PROPERTIES* textProps, UINT32 textLength, const(UINT16)* glyphIndices, const(DWRITE_SHAPING_GLYPH_PROPERTIES)* glyphProps, UINT32 glyphCount, IDWriteFontFace fontFace, FLOAT fontEmSize, FLOAT pixelsPerDip, const(DWRITE_MATRIX)* transform, BOOL useGdiNatural, BOOL isSideways, BOOL isRightToLeft, const(DWRITE_SCRIPT_ANALYSIS)* scriptAnalysis, const(WCHAR)* localeName, const(DWRITE_TYPOGRAPHIC_FEATURES)** features, const(UINT32)* featureRangeLengths, UINT32 featureRanges, FLOAT* glyphAdvances, DWRITE_GLYPH_OFFSET* glyphOffsets);
	}
}

struct DWRITE_GLYPH_RUN
{
	IDWriteFontFace fontFace;
	FLOAT fontEmSize;
	UINT32 glyphCount;
	const(UINT16)* glyphIndices;
	const(FLOAT)* glyphAdvances;
	const(DWRITE_GLYPH_OFFSET)* glyphOffsets;
	BOOL isSideways;
	UINT32 bidiLevel;
}

struct DWRITE_GLYPH_RUN_DESCRIPTION
{
	const(WCHAR)* localeName;
	const(WCHAR)* string;
	UINT32 stringLength;
	const(UINT16)* clusterMap;
	UINT32 textPosition;
}

struct DWRITE_UNDERLINE
{
	FLOAT width;
	FLOAT thickness;
	FLOAT offset;
	FLOAT runHeight;
	DWRITE_READING_DIRECTION readingDirection;
	DWRITE_FLOW_DIRECTION flowDirection;
	const(WCHAR)* localeName;
	DWRITE_MEASURING_MODE measuringMode;
}

struct DWRITE_STRIKETHROUGH
{
	FLOAT width;
	FLOAT thickness;
	FLOAT offset;
	DWRITE_READING_DIRECTION readingDirection;
	DWRITE_FLOW_DIRECTION flowDirection;
	const(WCHAR)* localeName;
	DWRITE_MEASURING_MODE measuringMode;
}

struct DWRITE_LINE_METRICS
{
	UINT32 length;
	UINT32 trailingWhitespaceLength;
	UINT32 newlineLength;
	FLOAT height;
	FLOAT baseline;
	BOOL isTrimmed;
}

struct DWRITE_CLUSTER_METRICS
{
	FLOAT width;
	UINT16 length;
	UINT16 bitfield;

	UINT16 canWrapLineAfter()
	{
		return 0b0000000000000001 & bitfield;
	}

	void canWrapLineAfter(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111111111110) | (0b0000000000000001 & a);
	}

	UINT16 isWhitespace()
	{
		return 0b0000000000000010 & bitfield;
	}

	void isWhitespace(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111111111101) | (0b0000000000000010 & a);
	}

	UINT16 isNewline()
	{
		return 0b0000000000000100 & bitfield;
	}

	void isNewline(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111111111011) | (0b0000000000000100 & a);
	}

	UINT16 isSoftHyphen()
	{
		return 0b0000000000001000 & bitfield;
	}

	void isSoftHyphen(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111111110111) | (0b0000000000001000 & a);
	}

	UINT16 isRightToLeft()
	{
		return 0b0000000000010000 & bitfield;
	}

	void isRightToLeft(UINT16 a)
	{
		bitfield = (bitfield & 0b1111111111101111) | (0b0000000000010000 & a);
	}

	UINT16 padding()
	{
		return 0b1111111111100000 & bitfield;
	}

	void padding(UINT16 a)
	{
		bitfield = (bitfield & 0b0000000000011111) | (0b1111111111100000 & a);
	}
}

struct DWRITE_TEXT_METRICS
{
	FLOAT left;
	FLOAT top;
	FLOAT width;
	FLOAT widthIncludingTrailingWhitespace;
	FLOAT height;
	FLOAT layoutWidth;
	FLOAT layoutHeight;
	UINT32 maxBidiReorderingDepth;
	UINT32 lineCount;
}

struct DWRITE_INLINE_OBJECT_METRICS
{
	FLOAT width;
	FLOAT height;
	FLOAT baseline;
	BOOL  supportsSideways;
}

struct DWRITE_OVERHANG_METRICS
{
	FLOAT left;
	FLOAT top;
	FLOAT right;
	FLOAT bottom;
}

struct DWRITE_HIT_TEST_METRICS
{
	UINT32 textPosition;
	UINT32 length;
	FLOAT left;
	FLOAT top;
	FLOAT width;
	FLOAT height;
	UINT32 bidiLevel;
	BOOL isText;
	BOOL isTrimmed;
}

extern (Windows)
{
	interface IDWriteInlineObject : IUnknown
	{
		HRESULT Draw(void* clientDrawingContext, IDWriteTextRenderer renderer, FLOAT originX, FLOAT originY, BOOL isSideways, BOOL isRightToLeft, IUnknown clientDrawingEffect);
		HRESULT GetMetrics(DWRITE_INLINE_OBJECT_METRICS* metrics);
		HRESULT GetOverhangMetrics(DWRITE_OVERHANG_METRICS* overhangs);
		HRESULT GetBreakConditions(DWRITE_BREAK_CONDITION* breakConditionBefore, DWRITE_BREAK_CONDITION* breakConditionAfter);
	}

	interface IDWritePixelSnapping : IUnknown
	{
		HRESULT IsPixelSnappingDisabled(void* clientDrawingContext, BOOL* isDisabled);
		HRESULT GetCurrentTransform(void* clientDrawingContext, DWRITE_MATRIX* transform);
		HRESULT GetPixelsPerDip(void* clientDrawingContext, FLOAT* pixelsPerDip);
	}

	interface IDWriteTextRenderer : IDWritePixelSnapping
	{
		HRESULT DrawGlyphRun(void* clientDrawingContext, FLOAT baselineOriginX, FLOAT baselineOriginY, DWRITE_MEASURING_MODE measuringMode, const(DWRITE_GLYPH_RUN)* glyphRun, const(DWRITE_GLYPH_RUN_DESCRIPTION)* glyphRunDescription, IUnknown clientDrawingEffect);
		HRESULT DrawUnderline(void* clientDrawingContext, FLOAT baselineOriginX, FLOAT baselineOriginY, const(DWRITE_UNDERLINE)* underline, IUnknown clientDrawingEffect);
		HRESULT DrawStrikethrough(void* clientDrawingContext, FLOAT baselineOriginX, FLOAT baselineOriginY, const(DWRITE_STRIKETHROUGH)* strikethrough, IUnknown clientDrawingEffect);
		HRESULT DrawInlineObject(void* clientDrawingContext, FLOAT originX, FLOAT originY, IDWriteInlineObject inlineObject, BOOL isSideways, BOOL isRightToLeft, IUnknown* clientDrawingEffect);
	}

	interface IDWriteTextLayout : IDWriteTextFormat
	{
		HRESULT SetMaxWidth(FLOAT maxWidth);
		HRESULT SetMaxHeight(FLOAT maxHeight);
		HRESULT SetFontCollection(IDWriteFontCollection fontCollection, DWRITE_TEXT_RANGE textRange);
		HRESULT SetFontFamilyName(const(WCHAR)* fontFamilyName, DWRITE_TEXT_RANGE textRange);
		HRESULT SetFontWeight(DWRITE_FONT_WEIGHT fontWeight, DWRITE_TEXT_RANGE textRange);
		HRESULT SetFontStyle(DWRITE_FONT_STYLE fontStyle, DWRITE_TEXT_RANGE textRange);
		HRESULT SetFontStretch(DWRITE_FONT_STRETCH fontStretch, DWRITE_TEXT_RANGE textRange);
		HRESULT SetFontSize(FLOAT fontSize, DWRITE_TEXT_RANGE textRange);
		HRESULT SetUnderline(BOOL hasUnderline, DWRITE_TEXT_RANGE textRange);
		HRESULT SetStrikethrough(BOOL hasStrikethrough, DWRITE_TEXT_RANGE textRange);
		HRESULT SetDrawingEffect(IUnknown drawingEffect, DWRITE_TEXT_RANGE textRange);
		HRESULT SetInlineObject(IDWriteInlineObject inlineObject, DWRITE_TEXT_RANGE textRange);
		HRESULT SetTypography(IDWriteTypography typography, DWRITE_TEXT_RANGE textRange);
		HRESULT SetLocaleName(const(WCHAR)* localeName, DWRITE_TEXT_RANGE textRange);
		FLOAT GetMaxWidth();
		FLOAT GetMaxHeight();
		HRESULT GetFontCollection(UINT32 currentPosition, IDWriteFontCollection* fontCollection, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetFontFamilyNameLength(UINT32 currentPosition, UINT32* nameLength, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetFontFamilyName(UINT32 currentPosition, WCHAR* fontFamilyName, UINT32 nameSize, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetFontWeight(UINT32 currentPosition, DWRITE_FONT_WEIGHT* fontWeight, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetFontStyle(UINT32 currentPosition, DWRITE_FONT_STYLE* fontStyle, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetFontStretch(DWRITE_FONT_STRETCH* fontStretch, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetFontSize(FLOAT* fontSize, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetUnderline(UINT32 currentPosition, BOOL* hasUnderline, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetStrikethrough(UINT32 currentPosition, BOOL* hasStrikethrough, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetDrawingEffect(UINT32 currentPosition, IUnknown* drawingEffect, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetInlineObject(UINT32 currentPosition, IDWriteInlineObject* inlineObject, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetTypography(UINT32 currentPosition, IDWriteTypography* typography, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetLocaleNameLength(UINT32 currentPosition, UINT32* nameLength, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT GetLocaleName(UINT32 currentPosition, WCHAR* localeName, UINT32 nameSize, DWRITE_TEXT_RANGE* textRange = null);
		HRESULT Draw(void* clientDrawingContext, IDWriteTextRenderer renderer, FLOAT originX, FLOAT originY);
		HRESULT GetLineMetrics(DWRITE_LINE_METRICS* lineMetrics, UINT32 maxLineCount, UINT32* actualLineCount);
		HRESULT GetMetrics(DWRITE_TEXT_METRICS* textMetrics);
		HRESULT GetOverhangMetrics(DWRITE_OVERHANG_METRICS* overhangs);
		HRESULT GetClusterMetrics(DWRITE_CLUSTER_METRICS* clusterMetrics, UINT32 maxClusterCount, UINT32* actualClusterCount);
		HRESULT DetermineMinWidth(FLOAT* minWidth);
		HRESULT HitTestPoint(FLOAT pointX, FLOAT pointY, BOOL* isTrailingHit, BOOL* isInside, DWRITE_HIT_TEST_METRICS* hitTestMetrics);
		HRESULT HitTestTextPosition(UINT32 textPosition, BOOL isTrailingHit, FLOAT* pointX, FLOAT* pointY, DWRITE_HIT_TEST_METRICS* hitTestMetrics);
		HRESULT HitTestTextRange(UINT32 textPosition, UINT32 textLength, FLOAT originX, FLOAT originY, DWRITE_HIT_TEST_METRICS* hitTestMetrics, UINT32 maxHitTestMetricsCount, UINT32* actualHitTestMetricsCount);
	}

	interface IDWriteBitmapRenderTarget : IUnknown
	{
		HRESULT DrawGlyphRun(FLOAT baselineOriginX, FLOAT baselineOriginY, DWRITE_MEASURING_MODE measuringMode, const(DWRITE_GLYPH_RUN)* glyphRun, IDWriteRenderingParams renderingParams, COLORREF textColor, RECT* blackBoxRect = null);
		HDC GetMemoryDC();
		FLOAT GetPixelsPerDip();
		HRESULT SetPixelsPerDip(FLOAT pixelsPerDip);
		HRESULT GetCurrentTransform(DWRITE_MATRIX* transform);
		HRESULT SetCurrentTransform(const(DWRITE_MATRIX)* transform);
		HRESULT GetSize(SIZE* size);
		HRESULT Resize(UINT32 width, UINT32 height);
	}

	interface IDWriteGdiInterop : IUnknown
	{
		HRESULT CreateFontFromLOGFONT(const(LOGFONTW)* logFont, IDWriteFont* font);
		HRESULT ConvertFontToLOGFONT(IDWriteFont font, LOGFONTW* logFont, BOOL* isSystemFont);
		HRESULT ConvertFontFaceToLOGFONT(IDWriteFontFace font, LOGFONTW* logFont);
		HRESULT CreateFontFaceFromHdc(HDC hdc, IDWriteFontFace* fontFace);
		HRESULT CreateBitmapRenderTarget(HDC hdc, UINT32 width, UINT32 height,IDWriteBitmapRenderTarget* renderTarget);
	}
}

enum DWRITE_TEXTURE_TYPE
{
	DWRITE_TEXTURE_ALIASED_1x1,
	DWRITE_TEXTURE_CLEARTYPE_3x1
}

immutable ubyte DWRITE_ALPHA_MAX = 255;

extern (Windows)
{
	interface IDWriteGlyphRunAnalysis : IUnknown
	{
		HRESULT GetAlphaTextureBounds(DWRITE_TEXTURE_TYPE textureType, RECT* textureBounds);
		HRESULT CreateAlphaTexture(DWRITE_TEXTURE_TYPE textureType, const(RECT)* textureBounds, BYTE* alphaValues, UINT32 bufferSize);
		HRESULT GetAlphaBlendParams(IDWriteRenderingParams renderingParams, FLOAT* blendGamma, FLOAT* blendEnhancedContrast, FLOAT* blendClearTypeLevel);
	}

	interface IDWriteFactory : IUnknown
	{
		HRESULT GetSystemFontCollection(IDWriteFontCollection* fontCollection, BOOL checkForUpdates = FALSE);
		HRESULT CreateCustomFontCollection(IDWriteFontCollectionLoader collectionLoader, const(void)* collectionKey, UINT32 collectionKeySize, IDWriteFontCollection* fontCollection);
		HRESULT RegisterFontCollectionLoader(IDWriteFontCollectionLoader fontCollectionLoader);
		HRESULT UnregisterFontCollectionLoader(IDWriteFontCollectionLoader fontCollectionLoader);
		HRESULT CreateFontFileReference(const(WCHAR)* filePath, const(FILETIME)* lastWriteTime, IDWriteFontFile* fontFile);
		HRESULT CreateCustomFontFileReference(const(void)* fontFileReferenceKey, UINT32 fontFileReferenceKeySize, IDWriteFontFileLoader fontFileLoader, IDWriteFontFile* fontFile);
		HRESULT CreateFontFace(DWRITE_FONT_FACE_TYPE fontFaceType, UINT32 numberOfFiles, const(IDWriteFontFile)* fontFiles, UINT32 faceIndex, DWRITE_FONT_SIMULATIONS fontFaceSimulationFlags, IDWriteFontFace* fontFace);
		HRESULT CreateRenderingParams(IDWriteRenderingParams* renderingParams);
		HRESULT CreateMonitorRenderingParams(HMONITOR monitor, IDWriteRenderingParams* renderingParams);
		HRESULT CreateCustomRenderingParams(FLOAT gamma, FLOAT enhancedContrast, FLOAT clearTypeLevel, DWRITE_PIXEL_GEOMETRY pixelGeometry, DWRITE_RENDERING_MODE renderingMode, IDWriteRenderingParams* renderingParams);
		HRESULT RegisterFontFileLoader(IDWriteFontFileLoader fontFileLoader);
		HRESULT UnregisterFontFileLoader(IDWriteFontFileLoader fontFileLoader);
		HRESULT CreateTextFormat(const(WCHAR)* fontFamilyName, IDWriteFontCollection fontCollection, DWRITE_FONT_WEIGHT fontWeight, DWRITE_FONT_STYLE fontStyle, DWRITE_FONT_STRETCH fontStretch, FLOAT fontSize, const(WCHAR)* localeName, IDWriteTextFormat* textFormat);
		HRESULT CreateTypography(IDWriteTypography* typography);
		HRESULT GetGdiInterop(IDWriteGdiInterop* gdiInterop);
		HRESULT CreateTextLayout(const(WCHAR)* str, UINT32 stringLength, IDWriteTextFormat* textFormat, FLOAT maxWidth, FLOAT maxHeight, IDWriteTextLayout* textLayout);
		HRESULT CreateGdiCompatibleTextLayout(const(WCHAR)* str, UINT32 stringLength, IDWriteTextFormat textFormat, FLOAT layoutWidth, FLOAT layoutHeight, FLOAT pixelsPerDip, const(DWRITE_MATRIX)* transform, BOOL useGdiNatural, IDWriteTextLayout* textLayout);
		HRESULT CreateEllipsisTrimmingSign(IDWriteTextFormat textFormat, IDWriteInlineObject* trimmingSign);
		HRESULT CreateTextAnalyzer(IDWriteTextAnalyzer* textAnalyzer);
		HRESULT CreateNumberSubstitution(DWRITE_NUMBER_SUBSTITUTION_METHOD substitutionMethod, const(WCHAR)* localeName, BOOL ignoreUserOverride, IDWriteNumberSubstitution* numberSubstitution);
		HRESULT CreateGlyphRunAnalysis(const(DWRITE_GLYPH_RUN)* glyphRun, FLOAT pixelsPerDip, const(DWRITE_MATRIX)* transform, DWRITE_RENDERING_MODE renderingMode, DWRITE_MEASURING_MODE measuringMode, FLOAT baselineOriginX, FLOAT baselineOriginY, IDWriteGlyphRunAnalysis* glyphRunAnalysis);
	}

	HRESULT DWriteCreateFactory (DWRITE_FACTORY_TYPE factoryType, REFIID iid, IUnknown* factory);
	alias HRESULT function (DWRITE_FACTORY_TYPE factoryType, REFIID iid, IUnknown* factory) DWriteCreateFactoryFunction;
}

immutable ushort FACILITY_DWRITE = 0x898;
immutable ushort DWRITE_ERR_BASE = 0x5000;

template MAKE_DWRITE_HR(ushort severity, ushort code)
{
	immutable HRESULT MAKE_DWRITE_HR = MAKE_HRESULT!(severity, FACILITY_DWRITE, DWRITE_ERR_BASE + code);
}

template MAKE_DWRITE_HR_ERR(ushort code)
{
	immutable HRESULT MAKE_DWRITE_HR_ERR = MAKE_DWRITE_HR!(SEVERITY_ERROR, code);
}

immutable HRESULT DWRITE_E_FILEFORMAT             = MAKE_DWRITE_HR_ERR!(0x000);
immutable HRESULT DWRITE_E_UNEXPECTED             = MAKE_DWRITE_HR_ERR!(0x001);
immutable HRESULT DWRITE_E_NOFONT                 = MAKE_DWRITE_HR_ERR!(0x002);
immutable HRESULT DWRITE_E_FILENOTFOUND           = MAKE_DWRITE_HR_ERR!(0x003);
immutable HRESULT DWRITE_E_FILEACCESS             = MAKE_DWRITE_HR_ERR!(0x004);
immutable HRESULT DWRITE_E_FONTCOLLECTIONOBSOLETE = MAKE_DWRITE_HR_ERR!(0x005);
immutable HRESULT DWRITE_E_ALREADYREGISTERED      = MAKE_DWRITE_HR_ERR!(0x006);


immutable GUID IID_IDWriteFontFileLoader       = {0x727cad4e, 0xd6af, 0x4c9e, [0x8a, 0x08, 0xd6, 0x95, 0xb1, 0x1c, 0xaa, 0x49]};
immutable GUID IID_IDWriteLocalFontFileLoader  = {0xb2d9f3ec, 0xc9fe, 0x4a11, [0xa2, 0xec, 0xd8, 0x62, 0x08, 0xf7, 0xc0, 0xa2]};
immutable GUID IID_IDWriteFontFileStream       = {0x6d4865fe, 0x0ab8, 0x4d91, [0x8f, 0x62, 0x5d, 0xd6, 0xbe, 0x34, 0xa3, 0xe0]};
immutable GUID IID_IDWriteFontFile             = {0x739d886a, 0xcef5, 0x47dc, [0x87, 0x69, 0x1a, 0x8b, 0x41, 0xbe, 0xbb, 0xb0]};
immutable GUID IID_IDWriteRenderingParams      = {0x2f0da53a, 0x2add, 0x47cd, [0x82, 0xee, 0xd9, 0xec, 0x34, 0x68, 0x8e, 0x75]};
immutable GUID IID_IDWriteFontFace             = {0x5f49804d, 0x7024, 0x4d43, [0xbf, 0xa9, 0xd2, 0x59, 0x84, 0xf5, 0x38, 0x49]};
immutable GUID IID_IDWriteFontCollectionLoader = {0xcca920e4, 0x52f0, 0x492b, [0xbf, 0xa8, 0x29, 0xc7, 0x2e, 0xe0, 0xa4, 0x68]};
immutable GUID IID_IDWriteFontFileEnumerator   = {0x72755049, 0x5ff7, 0x435d, [0x83, 0x48, 0x4b, 0xe9, 0x7c, 0xfa, 0x6c, 0x7c]};
immutable GUID IID_IDWriteLocalizedStrings     = {0x08256209, 0x099a, 0x4b34, [0xb8, 0x6d, 0xc2, 0x2b, 0x11, 0x0e, 0x77, 0x71]};
immutable GUID IID_IDWriteFontCollection       = {0xa84cee02, 0x3eea, 0x4eee, [0xa8, 0x27, 0x87, 0xc1, 0xa0, 0x2a, 0x0f, 0xcc]};
immutable GUID IID_IDWriteFontList             = {0x1a0d8438, 0x1d97, 0x4ec1, [0xae, 0xf9, 0xa2, 0xfb, 0x86, 0xed, 0x6a, 0xcb]};
immutable GUID IID_IDWriteFontFamily           = {0xda20d8ef, 0x812a, 0x4c43, [0x98, 0x02, 0x62, 0xec, 0x4a, 0xbd, 0x7a, 0xdd]};
immutable GUID IID_IDWriteFont                 = {0xacd16696, 0x8c14, 0x4f5d, [0x87, 0x7e, 0xfe, 0x3f, 0xc1, 0xd3, 0x27, 0x37]};
immutable GUID IID_IDWriteTextFormat           = {0x9c906818, 0x31d7, 0x4fd3, [0xa1, 0x51, 0x7c, 0x5e, 0x22, 0x5d, 0xb5, 0x5a]};
immutable GUID IID_IDWriteTypography           = {0x55f1112b, 0x1dc2, 0x4b3c, [0x95, 0x41, 0xf4, 0x68, 0x94, 0xed, 0x85, 0xb6]};
immutable GUID IID_IDWriteTextAnalyzer         = {0xb7e6163e, 0x7f46, 0x43b4, [0x84, 0xb3, 0xe4, 0xe6, 0x24, 0x9c, 0x36, 0x5d]};
immutable GUID IID_IDWriteInlineObject         = {0x8339FDE3, 0x106F, 0x47ab, [0x83, 0x73, 0x1C, 0x62, 0x95, 0xEB, 0x10, 0xB3]};
immutable GUID IID_IDWritePixelSnapping        = {0xeaf3a2da, 0xecf4, 0x4d24, [0xb6, 0x44, 0xb3, 0x4f, 0x68, 0x42, 0x02, 0x4b]};
immutable GUID IID_IDWriteTextRenderer         = {0xef8a8135, 0x5cc6, 0x45fe, [0x88, 0x25, 0xc5, 0xa0, 0x72, 0x4e, 0xb8, 0x19]};
immutable GUID IID_IDWriteTextLayout           = {0x53737037, 0x6d14, 0x410b, [0x9b, 0xfe, 0x0b, 0x18, 0x2b, 0xb7, 0x09, 0x61]};
immutable GUID IID_IDWriteBitmapRenderTarget   = {0x5e5a32a3, 0x8dff, 0x4773, [0x9f, 0xf6, 0x06, 0x96, 0xea, 0xb7, 0x72, 0x67]};
immutable GUID IID_IDWriteGdiInterop           = {0x1edd9491, 0x9853, 0x4299, [0x89, 0x8f, 0x64, 0x32, 0x98, 0x3b, 0x6f, 0x3a]};
immutable GUID IID_IDWriteGlyphRunAnalysis     = {0x7d97dbf7, 0xe085, 0x42d4, [0x81, 0xe3, 0x6a, 0x88, 0x3b, 0xde, 0xd1, 0x18]};
immutable GUID IID_IDWriteFactory              = {0xb859ee5a, 0xd838, 0x4b5b, [0xa2, 0xe8, 0x1a, 0xdc, 0x7d, 0x93, 0xdb, 0x48]};

