module jp.ku6.windows.win32api.uxtheme;

import jp.ku6.windows.win32api.w32api;
import jp.ku6.windows.win32api.windef;
import jp.ku6.windows.win32api.wingdi;
import jp.ku6.windows.win32api.commctrl;

alias HANDLE HTHEME;

static if (_WIN32_WINNT >= 0x0600)
{
	immutable uint MAX_THEMECOLOR = 64;
	immutable uint MAX_THEMESIZE = 64;
}

immutable uint OTD_FORCE_RECT_SIZING = 0x00000001;
immutable uint OTD_NONCLIENT         = 0x00000002;
immutable uint OTD_VALIDBITS         = OTD_FORCE_RECT_SIZING | OTD_NONCLIENT;

immutable uint DTBG_CLIPRECT        = 0x00000001;
immutable uint DTBG_DRAWSOLID       = 0x00000002;
immutable uint DTBG_OMITBORDER      = 0x00000004;
immutable uint DTBG_OMITCONTENT     = 0x00000008;
immutable uint DTBG_COMPUTINGREGION = 0x00000010;
immutable uint DTBG_MIRRORDC        = 0x00000020;
immutable uint DTBG_NOMIRROR        = 0x00000040;
immutable uint DTBG_VALIDBITS       = DTBG_CLIPRECT | DTBG_DRAWSOLID | DTBG_OMITBORDER | DTBG_OMITCONTENT | DTBG_COMPUTINGREGION | DTBG_MIRRORDC | DTBG_NOMIRROR;

struct DTBGOPTS
{
	DWORD dwSize;
	DWORD dwFlags;
	RECT rcClip;
}
alias DTBGOPTS* PDTBGOPTS;

immutable uint DTT_GRAYED          = 0x00000001;
immutable uint DTT_FLAGS2VALIDBITS = DTT_GRAYED;

enum THEMESIZE
{
	TS_MIN,
	TS_TRUE,
	TS_DRAW
}

immutable uint HTTB_BACKGROUNDSEG         = 0x00000000;
immutable uint HTTB_FIXEDBORDER           = 0x00000002;
immutable uint HTTB_CAPTION               = 0x00000004;
immutable uint HTTB_RESIZINGBORDER_LEFT   = 0x00000010;
immutable uint HTTB_RESIZINGBORDER_TOP    = 0x00000020;
immutable uint HTTB_RESIZINGBORDER_RIGHT  = 0x00000040;
immutable uint HTTB_RESIZINGBORDER_BOTTOM = 0x00000080;
immutable uint HTTB_SIZINGTEMPLATE        = 0x00000100;
immutable uint HTTB_SYSTEMSIZINGMARGINS   = 0x00000200;
immutable uint HTTB_RESIZINGBORDER        = HTTB_RESIZINGBORDER_LEFT | HTTB_RESIZINGBORDER_TOP | HTTB_RESIZINGBORDER_RIGHT | HTTB_RESIZINGBORDER_BOTTOM;

struct MARGINS
{
	int cxLeftWidth;
	int cxRightWidth;
	int cyTopHeight;
	int cyBottomHeight;
}
alias MARGINS* PMARGINS;

static if (_WIN32_WINNT >= 0x0600)
{
	immutable uint MAX_INTLIST_COUNT = 402;
}
else
{
	immutable uint MAX_INTLIST_COUNT = 10;
}

struct INTLIST
{
	int iValueCount;
	int[MAX_INTLIST_COUNT] iValues;
}
alias INTLIST* PINTLIST;

enum PROPERTYORIGIN
{
	PO_STATE,
	PO_PART,
	PO_CLASS,
	PO_GLOBAL,
	PO_NOTFOUND
}

immutable uint ETDT_DISABLE       = 0x00000001;
immutable uint ETDT_ENABLE        = 0x00000002;
immutable uint ETDT_USETABTEXTURE = 0x00000004;
immutable uint ETDT_ENABLETAB     = ETDT_ENABLE | ETDT_USETABTEXTURE;

static if (_WIN32_WINNT >= 0x0600)
{
	immutable uint ETDT_USEAEROWIZARDTABTEXTURE = 0x00000008;
	immutable uint ETDT_ENABLEAEROWIZARDTAB     = ETDT_ENABLE | ETDT_USEAEROWIZARDTABTEXTURE;
	immutable uint ETDT_VALIDBITS               = ETDT_DISABLE | ETDT_ENABLE | ETDT_USETABTEXTURE | ETDT_USEAEROWIZARDTABTEXTURE;
}

immutable uint STAP_ALLOW_NONCLIENT  = 1 << 0;
immutable uint STAP_ALLOW_CONTROLS   = 1 << 1;
immutable uint STAP_ALLOW_WEBCONTENT = 1 << 2;
immutable uint STAP_VALIDBITS        = STAP_ALLOW_NONCLIENT | STAP_ALLOW_CONTROLS | STAP_ALLOW_WEBCONTENT;

immutable wstring SZ_THDOCPROP_DISPLAYNAME   = "DisplayName"w;
immutable wstring SZ_THDOCPROP_CANONICALNAME = "ThemeName"w;
immutable wstring SZ_THDOCPROP_TOOLTIP       = "ToolTip"w;
immutable wstring SZ_THDOCPROP_AUTHOR        = "author"w;

immutable uint GBF_DIRECT    = 0x00000001;
immutable uint GBF_COPY      = 0x00000002;
immutable uint GBF_VALIDBITS = GBF_DIRECT | GBF_COPY;

immutable uint DTT_TEXTCOLOR    = 1 << 0;
immutable uint DTT_BORDERCOLOR  = 1 << 1;
immutable uint DTT_SHADOWCOLOR  = 1 << 2;
immutable uint DTT_SHADOWTYPE   = 1 << 3;
immutable uint DTT_SHADOWOFFSET = 1 << 4;
immutable uint DTT_BORDERSIZE   = 1 << 5;
immutable uint DTT_FONTPROP     = 1 << 6;
immutable uint DTT_COLORPROP    = 1 << 7;
immutable uint DTT_STATEID      = 1 << 8;
immutable uint DTT_CALCRECT     = 1 << 9;
immutable uint DTT_APPLYOVERLAY = 1 << 10;
immutable uint DTT_GLOWSIZE     = 1 << 11;
immutable uint DTT_CALLBACK     = 1 << 12;
immutable uint DTT_COMPOSITED   = 1 << 13;
immutable uint DTT_VALIDBITS    = DTT_TEXTCOLOR | DTT_BORDERCOLOR | DTT_SHADOWCOLOR | DTT_SHADOWTYPE | DTT_SHADOWOFFSET |  DTT_BORDERSIZE | DTT_FONTPROP | DTT_COLORPROP | DTT_STATEID | DTT_CALCRECT | DTT_APPLYOVERLAY | DTT_GLOWSIZE | DTT_COMPOSITED;

struct DTTOPTS
{
	DWORD dwSize;
	DWORD dwFlags;
	COLORREF crText;
	COLORREF crBorder;
	COLORREF crShadow;
	int iTextShadowType;
	POINT ptShadowOffset;
	int iBorderSize;
	int iFontPropId;
	int iColorPropId;
	int iStateId;
	BOOL fApplyOverlay;
	int iGlowSize;
	DTT_CALLBACK_PROC pfnDrawTextCallback;
	LPARAM lParam;
}
alias DTTOPTS* PDTTOPTS; 


static if (_WIN32_WINNT >= 0x0600)
{
	immutable uint DTPB_WINDOWDC          = 0x00000001;
	immutable uint DTPB_USECTLCOLORSTATIC = 0x00000002;
	immutable uint DTPB_USEERASEBKGND     = 0x00000004;

	enum WINDOWTHEMEATTRIBUTETYPE
	{
		WTA_NONCLIENT = 1
	}

	struct WTA_OPTIONS
	{
		DWORD dwFlags;
		DWORD dwMask;
	}
	alias WTA_OPTIONS* PWTA_OPTIONS;

	immutable uint WTNCA_NODRAWCAPTION = 0x00000001;
	immutable uint WTNCA_NODRAWICON    = 0x00000002;
	immutable uint WTNCA_NOSYSMENU     = 0x00000004;
	immutable uint WTNCA_NOMIRRORHELP  = 0x00000008;
	immutable uint WTNCA_VALIDBITS     = WTNCA_NODRAWCAPTION | WTNCA_NODRAWICON | WTNCA_NOSYSMENU | WTNCA_NOMIRRORHELP;

	HRESULT SetWindowThemeNonClientAttributes(HWND hwnd, DWORD dwMask, DWORD dwAttributes) 
	{
		WTA_OPTIONS wta;
		wta.dwFlags = dwAttributes;
		wta.dwMask = dwMask;
		return SetWindowThemeAttribute(hwnd, WINDOWTHEMEATTRIBUTETYPE.WTA_NONCLIENT, cast(void*)&(wta), wta.sizeof);
	}

	alias HANDLE HPAINTBUFFER;

	enum BP_BUFFERFORMAT
	{
		BPBF_COMPATIBLEBITMAP,
		BPBF_DIB,
		BPBF_TOPDOWNDIB,
		BPBF_TOPDOWNMONODIB,
		BPBF_COMPOSITED = BPBF_TOPDOWNDIB,
	}

	enum BP_ANIMATIONSTYLE
	{
		BPAS_NONE,
		BPAS_LINEAR,
		BPAS_CUBIC,
		BPAS_SINE
	}

	struct BP_ANIMATIONPARAMS
	{
		DWORD cbSize;
		DWORD dwFlags;
		BP_ANIMATIONSTYLE style;
		DWORD dwDuration;
	}
	alias BP_ANIMATIONPARAMS* PBP_ANIMATIONPARAMS;

	immutable ushort BPPF_ERASE     = 0x0001;
	immutable ushort BPPF_NOCLIP    = 0x0002;
	immutable ushort BPPF_NONCLIENT = 0x0004;

	struct BP_PAINTPARAMS
	{
		DWORD cbSize;
		DWORD dwFlags;
		const(RECT)* prcExclude;
		const(BLENDFUNCTION)* pBlendFunction;
	}
	alias BP_PAINTPARAMS* PBP_PAINTPARAMS;

	HRESULT BufferedPaintMakeOpaque(HPAINTBUFFER hBufferedPaint, const(RECT)* prc)
	{
		return BufferedPaintSetAlpha(hBufferedPaint, prc, 255);
	}

	alias HANDLE HANIMATIONBUFFER;
}

extern (Windows)
{
	BOOL BeginPanningFeedback(HWND hwnd); 
	BOOL UpdatePanningFeedback(HWND hwnd, LONG lTotalOverpanOffsetX, LONG lTotalOverpanOffsetY, BOOL fInInertia);
	BOOL EndPanningFeedback(HWND hwnd, BOOL fAnimateBack);
	HTHEME OpenThemeData(HWND hwnd, LPCWSTR pszClassList);
	HTHEME OpenThemeDataEx(HWND hwnd, LPCWSTR pszClassList, DWORD dwFlags);
	HRESULT CloseThemeData(HTHEME hTheme);
	HRESULT DrawThemeBackground(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCRECT pRect, LPCRECT pClipRect);
	HRESULT DrawThemeBackgroundEx(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCRECT pRect, const(DTBGOPTS)* pOptions);
	HRESULT DrawThemeText(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCWSTR pszText, int cchText, DWORD dwTextFlags, DWORD dwTextFlags2, LPCRECT pRect);
	HRESULT GetThemeBackgroundContentRect(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCRECT pBoundingRect, LPRECT pContentRect);
	HRESULT GetThemeBackgroundExtent(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCRECT pContentRect, LPRECT pExtentRect);
	HRESULT GetThemeBackgroundRegion(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCRECT pRect, HRGN* pRegion);
	HRESULT GetThemePartSize(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCRECT prc, THEMESIZE eSize, SIZE* psz);
	HRESULT GetThemeTextExtent(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCWSTR pszText, int cchCharCount, DWORD dwTextFlags, LPCRECT pBoundingRect, LPRECT pExtentRect);
	HRESULT GetThemeTextMetrics(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, TEXTMETRICW* ptm);
	HRESULT HitTestThemeBackground(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, DWORD dwOptions, LPCRECT pRect, HRGN hrgn, POINT ptTest, WORD* pwHitTestCode);
	HRESULT DrawThemeEdge(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCRECT pDestRect, UINT uEdge, UINT uFlags, LPRECT pContentRect);
	HRESULT DrawThemeIcon(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCRECT pRect, HIMAGELIST himl, int iImageIndex);
	BOOL IsThemePartDefined(HTHEME hTheme, int iPartId, int iStateId);
	BOOL IsThemeBackgroundPartiallyTransparent(HTHEME hTheme, int iPartId, int iStateId);
	HRESULT GetThemeColor(HTHEME hTheme, int iPartId, int iStateId, int iPropId, COLORREF* pColor);
	HRESULT GetThemeMetric(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, int iPropId, int* piVal);
	HRESULT GetThemeString(HTHEME hTheme, int iPartId, int iStateId, int iPropId, LPWSTR pszBuff, int cchMaxBuffChars);
	HRESULT GetThemeBool(HTHEME hTheme, int iPartId, int iStateId, int iPropId, BOOL* pfVal);
	HRESULT GetThemeInt(HTHEME hTheme, int iPartId, int iStateId, int iPropId, int* piVal);
	HRESULT GetThemeEnumValue(HTHEME hTheme, int iPartId, int iStateId, int iPropId, int* piVal);
	HRESULT GetThemePosition(HTHEME hTheme, int iPartId, int iStateId, int iPropId, POINT* pPoint);
	HRESULT GetThemeFont(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, int iPropId, LOGFONTW* pFont);
	HRESULT GetThemeRect(HTHEME hTheme, int iPartId, int iStateId, int iPropId, LPRECT pRect);
	HRESULT GetThemeMargins(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, int iPropId, LPCRECT prc, MARGINS* pMargins);
	HRESULT GetThemeIntList(HTHEME hTheme, int iPartId, int iStateId, int iPropId, INTLIST* pIntList);
	HRESULT GetThemePropertyOrigin(HTHEME hTheme, int iPartId, int iStateId, int iPropId, PROPERTYORIGIN* pOrigin);
	HRESULT SetWindowTheme(HWND hwnd, LPCWSTR pszSubAppName, LPCWSTR pszSubIdList);
	HRESULT GetThemeFilename(HTHEME hTheme, int iPartId, int iStateId, int iPropId, LPWSTR pszThemeFileName, int cchMaxBuffChars);
	COLORREF GetThemeSysColor(HTHEME hTheme, int iColorId);
	HBRUSH GetThemeSysColorBrush(HTHEME hTheme, int iColorId);
	BOOL GetThemeSysBool(HTHEME hTheme, int iBoolId);
	int GetThemeSysSize(HTHEME hTheme, int iSizeId);
	HRESULT GetThemeSysFont(HTHEME hTheme, int iFontId, LOGFONTW* plf);
	HRESULT GetThemeSysString(HTHEME hTheme, int iStringId, LPWSTR pszStringBuff, int cchMaxStringChars);
	HRESULT GetThemeSysInt(HTHEME hTheme, int iIntId, int* piValue);
	BOOL IsThemeActive();
	BOOL IsAppThemed();
	HTHEME GetWindowTheme(HWND hwnd);
	HTHEME EnableThemeDialogTexture(HWND hwnd, DWORD dwFlags);
	BOOL IsThemeDialogTextureEnabled(HWND hwnd);
	DWORD GetThemeAppProperties();
	void SetThemeAppProperties(DWORD dwFlags);
	HRESULT GetCurrentThemeName(LPWSTR pszThemeFileName, int cchMaxNameChars, LPWSTR pszColorBuff, int cchMaxColorChars, LPWSTR pszSizeBuff, int cchMaxSizeChars);
	HRESULT GetThemeDocumentationProperty(LPCWSTR pszThemeName, LPCWSTR pszPropertyName, LPWSTR pszValueBuff, int cchMaxValChars);
	HRESULT DrawThemeParentBackground(HWND hwnd, HDC hdc, const(RECT)* prc);
	HRESULT EnableTheming(BOOL fEnable);

	static if (_WIN32_WINNT >= 0x0600)
	{
		HRESULT DrawThemeParentBackgroundEx(HWND hwnd, HDC hdc, DWORD dwFlags, const(RECT)* prc);
		HRESULT SetWindowThemeAttribute(HWND hwnd, WINDOWTHEMEATTRIBUTETYPE eAttribute, PVOID pvAttribute, DWORD cbAttribute);
		HRESULT DrawThemeTextEx(HTHEME hTheme, HDC hdc, int iPartId, int iStateId, LPCWSTR pszText, int cchText, DWORD dwTextFlags, LPRECT pRect, const(DTTOPTS)* pOptions);
		HRESULT GetThemeBitmap(HTHEME hTheme, int iPartId, int iStateId, int iPropId, ULONG dwFlags, HBITMAP* phBitmap);
		HRESULT GetThemeStream(HTHEME hTheme, int iPartId, int iStateId, int iPropId, VOID** ppvStream, DWORD* pcbStream, HINSTANCE hInst);
		HRESULT BufferedPaintInit();
		HRESULT BufferedPaintUnInit();
		HPAINTBUFFER BeginBufferedPaint(HDC hdcTarget, const(RECT)* prcTarget, BP_BUFFERFORMAT dwFormat, BP_PAINTPARAMS *pPaintParams, HDC* phdc);
		HRESULT EndBufferedPaint(HPAINTBUFFER hBufferedPaint, BOOL fUpdateTarget);
		HRESULT GetBufferedPaintTargetRect(HPAINTBUFFER hBufferedPaint, RECT* prc);
		HDC GetBufferedPaintTargetDC(HPAINTBUFFER hBufferedPaint);
		HDC GetBufferedPaintDC(HPAINTBUFFER hBufferedPaint);
		HRESULT GetBufferedPaintBits(HPAINTBUFFER hBufferedPaint, RGBQUAD** ppbBuffer, int* pcxRow);
		HRESULT BufferedPaintClear(HPAINTBUFFER hBufferedPaint, const(RECT)* prc);
		HRESULT BufferedPaintSetAlpha(HPAINTBUFFER hBufferedPaint, const(RECT)* prc, BYTE alpha);
		HRESULT BufferedPaintStopAllAnimations(HWND hwnd);
		HANIMATIONBUFFER BeginBufferedAnimation(HWND hwnd, HDC hdcTarget, const(RECT)* prcTarget, BP_BUFFERFORMAT dwFormat, BP_PAINTPARAMS* pPaintParams, BP_ANIMATIONPARAMS* pAnimationParams, HDC* phdcFrom, HDC* phdcTo);
		HRESULT EndBufferedAnimation(HANIMATIONBUFFER hbpAnimation, BOOL fUpdateTarget);
		BOOL BufferedPaintRenderAnimation(HWND hwnd, HDC hdcTarget);
		BOOL IsCompositionActive();
		HRESULT GetThemeTransitionDuration(HTHEME hTheme, int iPartId, int iStateIdFrom, int iStateIdTo, int iPropId, DWORD* pdwDuration);  
	}

	alias int function(HDC hdc, LPWSTR pszText, int cchText, LPRECT prc, UINT dwFlags, LPARAM lParam) DTT_CALLBACK_PROC;
}